@@ -128,30 +128,35 @@ gsf_input_class_init (GObjectClass *gobject_class)
 					 g_param_spec_string ("name", "Name",
 							      "The Input's Name",
 							      NULL,
+							      GSF_PARAM_STATIC |
 							      G_PARAM_READABLE));
 	g_object_class_install_property (gobject_class,
 					 PROP_SIZE,
 					 g_param_spec_int64 ("size", "Size",
 							     "The Input's Size",
 							     0, G_MAXINT64, 0,
+							     GSF_PARAM_STATIC |
 							     G_PARAM_READABLE));
 	g_object_class_install_property (gobject_class,
 					 PROP_EOF,
 					 g_param_spec_boolean ("eof", "OEF",
 							       "End Of File",
 							       FALSE,
+							       GSF_PARAM_STATIC |
 							       G_PARAM_READABLE));
 	g_object_class_install_property (gobject_class,
 					 PROP_REMAINING,
 					 g_param_spec_int64 ("remaining", "Remaining",
 							     "Amount of Data Remaining",
 							     0, G_MAXINT64, 0,
+							     GSF_PARAM_STATIC |
 							     G_PARAM_READABLE));
 	g_object_class_install_property (gobject_class,
 					 PROP_POS,
 					 g_param_spec_int64 ("position", "Position",
 							     "The Output's Current Position",
 							     0, G_MAXINT64, 0,
+							     GSF_PARAM_STATIC |
 							     G_PARAM_READABLE));
 }
 
@@ -206,14 +211,14 @@ gsf_input_dup (GsfInput *input, GError **err)
 	if (dst != NULL) {
 		if (dst->size != input->size) {
 			if (err != NULL)
-				*err = g_error_new (gsf_input_error (), 0,
+				*err = g_error_new (gsf_input_error_id (), 0,
 						    "Duplicate size mismatch");
 			g_object_unref (dst);
 			return NULL;
 		}
 		if (gsf_input_seek (dst, input->cur_offset, G_SEEK_SET)) {
 			if (err != NULL)
-				*err = g_error_new (gsf_input_error (), 0,
+				*err = g_error_new (gsf_input_error_id (), 0,
 						    "Seek failed");
 			g_object_unref (dst);
 			return NULL;
@@ -228,6 +233,21 @@ gsf_input_dup (GsfInput *input, GError **err)
 	return dst;
 }
 
+/**
+ * gsf_input_open_sibling :
+ * @input : The input
+ *
+ * Attempts to open a 'sibling' of @input.  The caller is responsible for
+ * managing the resulting object.
+ *
+ * Returns :  the size or -1 on error
+ **/
+GsfInput *
+gsf_input_sibling (GsfInput const *input, char const *name, GError **err)
+{
+	return NULL;
+}
+
 /**
  * gsf_input_size :
  * @input : The input
@@ -459,18 +479,30 @@ gsf_input_seek_emulate (GsfInput *input, gsf_off_t pos)
 
 /****************************************************************************/
 
+/**
+ * gsf_input_error_id :
+ *
+ * Returns : A utility quark to flag a GError as being an input problem.
+ */
+GQuark 
+gsf_input_error_id (void)
+{
+	static GQuark quark;
+	if (!quark)
+		quark = g_quark_from_static_string ("gsf_input_error_id");
+	return quark;
+}
+
 /**
  * gsf_input_error :
  *
+ * Deprecated in 1.12.0
  * Returns : A utility quark to flag a GError as being an input problem.
  */
 GQuark 
 gsf_input_error (void)
 {
-	static GQuark quark;
-	if (!quark)
-		quark = g_quark_from_static_string ("gsf_input_error");
-	return quark;
+	return gsf_input_error_id ();
 }
 
 /****************************************************************************/
