@@ -24,6 +24,7 @@
 #include <gsf/gsf-output-impl.h>
 #include <gsf/gsf-impl-utils.h>
 #include <gsf/gsf-utils.h>
+#include <glib/gstdio.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -34,30 +35,19 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-
 #ifdef G_OS_WIN32
-
-#ifndef S_IRUSR
-#define S_IRUSR 04
-#define S_IWUSR 02
-#define W_OK    00  /* existance mode only */
-typedef int mode_t;
-#endif
-
-#define S_IRGRP S_IRUSR
-#define S_IROTH S_IRUSR
-#define S_ISGID 0
-
-#define getuid() 0
-#define getgid() 0
-#define chown(filename, uid, gid) 0
-
+#include <wchar.h>
+#include <direct.h>
+#include <glib/gwin32.h>
 #ifdef HAVE_IO_H
 #include <io.h>
 #endif
-
 #endif /* G_OS_WIN32 */
 
+#ifndef W_OK
+#define W_OK 2
+#endif
+
 static GObjectClass *parent_class;
 
 struct _GsfOutputStdio {
@@ -74,96 +64,108 @@ typedef struct {
 } GsfOutputStdioClass;
 
 static int
-rename_wrapper (const char *oldfilename, const char *newfilename)
+rename_wrapper (char const *oldfilename, char const *newfilename)
 {
-	int result = rename (oldfilename, newfilename);
+	int result = g_rename (oldfilename, newfilename);
 #ifdef G_OS_WIN32
 	if (result) {
 		/* Win32's rename does not unlink the target.  */
-		(void)unlink (newfilename);
-		result = rename (oldfilename, newfilename);
+		(void)g_unlink (newfilename);
+		result = g_rename (oldfilename, newfilename);
 	}
 #endif
 	return result;
 }
 
+/* g_access would be nice here.  */
+static gboolean
+gsf_access (char const *filename, int what)
+{
+#ifdef G_OS_WIN32
+	int retval;
+	int save_errno;
+	if (G_WIN32_HAVE_WIDECHAR_API ()) {
+		wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
+		if (wfilename == NULL) {
+			errno = EINVAL;
+			return -1;
+		}
+
+		retval = _waccess (wfilename, what);
+		save_errno = errno;
+
+		g_free (wfilename);
+      
+		errno = save_errno;
+	} else {
+		gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
+		if (cp_filename == NULL) {
+			errno = EINVAL;
+			return -1;
+		}
+
+		retval = _access (cp_filename, what);
+		save_errno = errno;
+
+		g_free (cp_filename);
+
+		errno = save_errno;
+	}
+	return retval;
+#else
+	return access (filename, what);
+#endif
+}
+
+
 
 #define GSF_MAX_LINK_LEVEL 256
-#ifdef MAXPATHLEN
-#define GSF_MAX_PATH_LEN  MAXPATHLEN
-#elif defined (PATH_MAX)
-#define GSF_MAX_PATH_LEN  PATH_MAX
-#else
-#define GSF_MAX_PATH_LEN  2048
-#endif
 
-/* Does readlink() recursively until we find a real filename. */
+/* Calls g_file_read_link() until we find a real filename. */
 static char *
 follow_symlinks (char const *filename, GError **error)
 {
-#ifdef HAVE_READLINK
-	gchar *followed_filename;
-	gint link_count;
+	gchar *followed_filename, *link;
+	gint link_count = 0;
 
 	g_return_val_if_fail (filename != NULL, NULL);
-	g_return_val_if_fail (strlen (filename) + 1 <= GSF_MAX_PATH_LEN, NULL);
 
 	followed_filename = g_strdup (filename);
-	link_count = 0;
 
-	while (link_count++ < GSF_MAX_LINK_LEVEL) {
-		char linkname[GSF_MAX_PATH_LEN];
-		int len = readlink (followed_filename, linkname, GSF_MAX_PATH_LEN - 1);
-
-		if (len == -1) {
-			switch (errno) {
-			case EINVAL: /* Not a symlink.  */
-			case ENOSYS: /* Surely not a symlink.  */
-			case ENOENT: /* No such file.  */
-				return followed_filename;
-
-			default:
-				if (error)
-					*error = g_error_new_literal 
-						(gsf_output_error_id (), errno,
-						 g_strerror (errno));
-				g_free (followed_filename);
-				return NULL;
-			}
-		}
-
-		linkname[len] = '\0';
-
-		if (g_path_is_absolute (linkname)) {
+	while ((link = g_file_read_link (followed_filename, NULL)) != NULL &&
+	  ++link_count <= GSF_MAX_LINK_LEVEL) {
+		if (g_path_is_absolute (link)) {
 			g_free (followed_filename);
-			followed_filename = g_strdup (linkname);
+			followed_filename = link;
 		} else {
-			/*
-			 * If the linkname is not an absolute path name, append
+			/* If the linkname is not an absolute path name, append
 			 * it to the directory name of the followed filename.  E.g.
 			 * we may have /foo/bar/baz.lnk -> eek.txt, which really
-			 * is /foo/bar/eek.txt.
-			 */
-
-			char *dir = g_path_get_dirname (followed_filename);
-
+			 * is /foo/bar/eek.txt.  */
+			gchar *dir = g_path_get_dirname (followed_filename);
 			g_free (followed_filename);
-			followed_filename = g_build_filename (dir, linkname, NULL);
+			followed_filename = g_build_filename (dir, link, NULL);
 			g_free (dir);
+			g_free (link);
 		}
 	}
 
+	if (link == NULL)
+		return followed_filename;
+
 	/* Too many symlinks */
-	if (error)
-		*error = g_error_new_literal (gsf_output_error_id (), ELOOP,
-					      g_strerror (ELOOP));
-
+	if (error != NULL) {
+#ifdef ELOOP
+		int err = ELOOP;
+#else
+		/* We have links, but not ELOOP.  Very strange.  */
+		int err = EINVAL;
+#endif
+		*error = g_error_new_literal (gsf_output_error_id (), err,
+					      g_strerror (err));
+	}
+	g_free (followed_filename);
 	return NULL;
-#else  /* !HAVE_READLINK */
-
-	return g_strdup (filename);
-
-#endif /* !HAVE_READLINK */
 }
 
 /**
@@ -192,14 +194,31 @@ gsf_output_stdio_new (char const *filename, GError **err)
 	/* Get the directory in which the real filename lives */
 	dirname = g_path_get_dirname (real_filename);
 
-	if (stat (real_filename, &st) == 0) {
-		/* FIXME: use eaccess if available.  */
+	if (g_stat (real_filename, &st) == 0) {
+		if (!S_ISREG (st.st_mode)) {
+			if (err != NULL) {
+				char *dname = g_filename_display_name
+					(real_filename);
+				*err = g_error_new (gsf_output_error_id (), 0,
+						    "%s: Is not a regular file",
+						    dname);
+				g_free (dname);
+			}
+			goto failure;
+		}
+
 		/* FIXME? Race conditions en masse.  */
-		if (access (real_filename, W_OK) != 0) {
-			if (err != NULL)
-				*err = g_error_new_literal
+		if (gsf_access (real_filename, W_OK) == -1) {
+			if (err != NULL) {
+				int save_errno = errno;
+				char *dname = g_filename_display_name
+					(real_filename);
+				*err = g_error_new
 					(gsf_output_error_id (), errno,
-					 g_strerror (errno));
+					 "%s: %s",
+					 dname, g_strerror (save_errno));
+				g_free (dname);
+			}
 			goto failure;
 		}
 	} else {
@@ -209,22 +228,21 @@ gsf_output_stdio_new (char const *filename, GError **err)
 		 */
 
 		struct stat dir_st;
-		int result;
+
+		memset (&st, 0, sizeof (st));
 
 		/* Use default permissions */
 		st.st_mode = 0666;  fixup_mode = TRUE;
+#ifdef HAVE_CHOWN
 		st.st_uid = getuid ();
 
-		/* Used to set create_backup_copy = FALSE, but
-		   creating backup copies on this level is a 
-		   horrible mistake. -DAL- */
-
-		result = stat (dirname, &dir_st);
-
-		if (result == 0 && (dir_st.st_mode & S_ISGID))
+		if (g_stat (dirname, &dir_st) == 0 &&
+		    S_ISDIR (st.st_mode) &&
+		    (dir_st.st_mode & S_ISGID))
 			st.st_gid = dir_st.st_gid;
 		else
 			st.st_gid = getgid ();
+#endif
 	}
 
 	/* Save to a temporary file.  We set the umask because some (buggy)
@@ -240,10 +258,16 @@ gsf_output_stdio_new (char const *filename, GError **err)
 		st.st_mode &= ~saved_umask;
 
 	if (fd < 0 || NULL == (file = fdopen (fd, "wb"))) {
-		if (err != NULL)
-			*err = g_error_new_literal
+		if (err != NULL) {
+			int save_errno = errno;
+			char *dname = g_filename_display_name
+				(temp_filename);
+			*err = g_error_new
 				(gsf_output_error_id (), errno,
-				 g_strerror (errno));
+				 "%s: %s",
+				 dname, g_strerror (save_errno));
+			g_free (dname);
+		}
 		goto failure;
 	}
 
@@ -260,7 +284,7 @@ gsf_output_stdio_new (char const *filename, GError **err)
 
 	return GSF_OUTPUT (stdio);
 
-failure :
+ failure:
 	g_free (temp_filename);
 	g_free (real_filename);
 	g_free (dirname);
@@ -297,6 +321,33 @@ gsf_output_stdio_new_FILE (char const *filename, FILE *file, gboolean keep_open)
 	return GSF_OUTPUT (stdio);
 }
 
+static gboolean
+close_file_helper (GsfOutputStdio *stdio, gboolean seterr)
+{
+	gboolean res = (0 == fclose (stdio->file));
+	stdio->file = NULL;
+	if (!res && seterr)
+		gsf_output_set_error (GSF_OUTPUT (stdio), errno,
+				      "Failed to close file: %s",
+				      g_strerror (errno));
+	return res;
+}
+
+static gboolean
+unlink_file_helper (GsfOutputStdio *stdio)
+{
+	if (!stdio->temp_filename)
+		return TRUE;
+
+	if (g_unlink (stdio->temp_filename) == 0) {
+		g_free (stdio->temp_filename);
+		stdio->temp_filename = NULL;
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static gboolean
 gsf_output_stdio_close (GsfOutput *output)
 {
@@ -307,6 +358,17 @@ gsf_output_stdio_close (GsfOutput *output)
 	if (stdio->file == NULL)
 		return FALSE;
 
+	if (gsf_output_error (output)) {
+		res = TRUE;
+		if (!stdio->keep_open && !close_file_helper (stdio, FALSE))
+			res = FALSE;
+
+		if (!unlink_file_helper (stdio))
+			res = FALSE;
+
+		return res;
+	}
+
 	if (stdio->keep_open) {
 		gboolean res = (0 == fflush (stdio->file));
 		if (!res)
@@ -316,21 +378,13 @@ gsf_output_stdio_close (GsfOutput *output)
 		return res;
 	}
 
-	res = (0 == fclose (stdio->file));
-	stdio->file = NULL;
+	res = close_file_helper (stdio, TRUE);
 
 	/* short circuit our when dealing with raw FILE */
-	if (NULL == stdio->real_filename) {
-		if (!res)
-			gsf_output_set_error (output, errno,
-				"Failed to close file.");
+	if (!stdio->real_filename)
 		return res;
-	}
-
 	if (!res) {
-		gsf_output_set_error (output, errno,
-				      "Failed to close temporary file.");
-		unlink (stdio->temp_filename);
+		unlink_file_helper (stdio);
 		return FALSE;
 	}
 
@@ -340,13 +394,13 @@ gsf_output_stdio_close (GsfOutput *output)
 		backup_filename = g_strconcat (stdio->real_filename, ".bak", NULL);
 		result = rename_wrapper (stdio->real_filename, backup_filename);
 		if (result != 0) {
-			char *utf8name = gsf_filename_to_utf8 (backup_filename, TRUE);
+			char *utf8name = g_filename_display_name (backup_filename);
 			gsf_output_set_error (output, errno,
-				"Could not backup the original as %s.",
-				utf8name);
+					      "Could not backup the original as %s.",
+					      utf8name);
 			g_free (utf8name);
 			g_free (backup_filename);
-			unlink (stdio->temp_filename);
+			g_unlink (stdio->temp_filename);
 			return FALSE;
 		}
 	}
@@ -363,7 +417,9 @@ gsf_output_stdio_close (GsfOutput *output)
 		 * can do here, I'm afraid.  The final data is saved anyways.
 		 * Note the order: mode, uid+gid, gid, uid, mode.
 		 */
+#warning "We need g_chmod in gstdio.h for this"
 		chmod (stdio->real_filename, stdio->st.st_mode);
+#ifdef HAVE_CHOWN
 		if (chown (stdio->real_filename,
 			   stdio->st.st_uid,
 			   stdio->st.st_gid)) {
@@ -372,6 +428,7 @@ gsf_output_stdio_close (GsfOutput *output)
 			chown (stdio->real_filename, stdio->st.st_uid, -1);
 		}
 		chmod (stdio->real_filename, stdio->st.st_mode);
+#endif
 	}
 
 	g_free (backup_filename);
