@@ -79,30 +79,28 @@ gsf_output_get_property (GObject     *object,
 }
 
 static void
-gsf_output_finalize (GObject *obj)
+gsf_output_dispose (GObject *obj)
 {
-	GsfOutput *output;
+	GsfOutput *output = GSF_OUTPUT (obj);
 
-	output = GSF_OUTPUT (obj);
-
-	/* it is too late to close things, we are partially destroyed.
-	 * Keep this as a warning for silly mistakes
-	 */
-	if (!output->is_closed)
-		g_warning ("unrefing an unclosed stream");
+	if (!output->is_closed) {
+		/* g_warning ("Disposing of an unclosed stream"); */
+		gsf_output_close (output);
+	}
 
 	g_free (output->name);
 	output->name = NULL;
 	g_free (output->printf_buf);
 	output->printf_buf = NULL;
 
+	g_clear_error (&output->err);
+
 	if (output->container != NULL) {
 		g_object_unref (G_OBJECT (output->container));
 		output->container = NULL;
 	}
-	g_clear_error (&output->err);
 
-	G_OBJECT_CLASS (parent_class)->finalize (obj);
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 static void
@@ -126,7 +124,7 @@ gsf_output_class_init (GObjectClass *gobject_class)
 {
 	GsfOutputClass  *output_class  = GSF_OUTPUT_CLASS (gobject_class);
 
-	gobject_class->finalize     = gsf_output_finalize;
+	gobject_class->dispose      = gsf_output_dispose;
 	gobject_class->set_property = gsf_output_set_property;
 	gobject_class->get_property = gsf_output_get_property;
 	output_class->Vprintf       = gsf_output_real_vprintf;
@@ -138,24 +136,28 @@ gsf_output_class_init (GObjectClass *gobject_class)
 					 g_param_spec_string ("name", "Name",
 							      "The Output's Name",
 							      NULL,
+							      GSF_PARAM_STATIC |
 							      G_PARAM_READABLE));
 	g_object_class_install_property (gobject_class,
 					 PROP_SIZE,
 					 g_param_spec_int64 ("size", "Size",
 							     "The Output's Size",
 							     0, G_MAXINT64, 0,
+							     GSF_PARAM_STATIC |
 							     G_PARAM_READABLE));
 	g_object_class_install_property (gobject_class,
 					 PROP_POS,
 					 g_param_spec_int64 ("position", "Position",
 							     "The Output's Current Position",
 							     0, G_MAXINT64, 0,
+							     GSF_PARAM_STATIC |
 							     G_PARAM_READABLE));
 	g_object_class_install_property (gobject_class,
 					 PROP_CLOSED,
-					 g_param_spec_boolean ("is_closed", "Is Closed",
+					 g_param_spec_boolean ("is-closed", "Is Closed",
 							       "Whether the Output is Closed",
 							       FALSE,
+							       GSF_PARAM_STATIC |
 							       G_PARAM_READABLE));
 }
 
@@ -369,15 +371,14 @@ gsf_output_set_name (GsfOutput *output, char const *name)
 	g_return_val_if_fail (GSF_IS_OUTPUT (output), FALSE);
 
 	buf = g_strdup (name);
-	if (output->name != NULL)
-		g_free (output->name);
+	g_free (output->name);
 	output->name = buf;
 	return TRUE;
 }
 
 /**
  * gsf_output_set_name_from_filename :
- * @input : the output stream
+ * @output : the output stream
  * @filename : the (fs-sys encoded) filename
  *
  * <note>This is a utility routine that should only be used by derived
